@@ -348,7 +348,8 @@ int SendRead10(CMDDRIVE *drive, DWORD lba, WORD len, DWORD buflen)
 	return SendCmd(drive, cdb, buflen, REQ_DATAIN);
 }
 
-int SendWrite10(CMDDRIVE *drive, DWORD lba, WORD len, DWORD buflen)
+int SendWrite10(CMDDRIVE *drive, DWORD lba, WORD len, DWORD buflen,
+				BOOL retry)
 {
 	BYTE cdb[12];
 	int ret;
@@ -357,14 +358,19 @@ int SendWrite10(CMDDRIVE *drive, DWORD lba, WORD len, DWORD buflen)
 	cdb[0] = CMD_WRITE10;
 	Set4bytes(cdb+2, lba);
 	Set2bytes(cdb+7, len);
-	while(TRUE){
+	if(retry){
+	  while(TRUE){
 		ret = SendCmd(drive, cdb, buflen, REQ_DATAOUT);
 		if(ret!=RET_CMDERR)
-			break;
+		  break;
 		if(!(SD_ASC(drive)==0x04 && SD_ASCQ(drive)==0x08))
-			break;
+		  break;
+	  }
+	  return ret;
+	}
+	else{
+	  return SendCmd(drive, cdb, buflen, REQ_DATAOUT);
 	}
-	return ret;
 }
 
 int SendSeek10(CMDDRIVE *drive, DWORD lba)
@@ -393,6 +399,16 @@ int SendSendCueSheet(CMDDRIVE *drive, void *cuesheet, DWORD len)
 	return SendCmd(drive, cdb, len, REQ_DATAOUT);
 }
 
+int SendReadBufferCapacity(CMDDRIVE *drive, BYTE block)
+{
+	BYTE cdb[12];
+
+	memset(cdb, 0, sizeof(cdb));
+	cdb[0] = CMD_READ_BUFFER_CAPACITY;
+	cdb[1] = (BYTE)(block ? 1:0);
+	Set2bytes(cdb+7, 12);
+	return SendCmd(drive, cdb, 12, REQ_DATAIN);
+}
 
 
 int SendLongRead12(CMDDRIVE *drive, DWORD lba, DWORD len, DWORD buflen,
@@ -500,6 +516,7 @@ int CheckReady(CMDDRIVE *drive, BOOL bWaitDisc)
 		if(SD_SENSEKEY(drive)==0x02 &&
 		   SD_ASC(drive)==0x04 &&
 		   SD_ASCQ(drive)==0x01){
+		  UIDispInfo("ł̂҂Ă܂");
 			  /* becoming ready */
 		}
 		else if(SD_SENSEKEY(drive)==0x06 &&
@@ -525,6 +542,7 @@ int CheckReady(CMDDRIVE *drive, BOOL bWaitDisc)
 			if(bEjected==FALSE){
 			  OpenTray(drive);
 			  bEjected=TRUE;
+			  UIDispInfo("fBXN}Ă");
 			}
 		  }
 		  else{
