@@ -132,9 +132,11 @@ extern int SendSetStreaming(CMDDRIVE *drive);
 extern int SendSynchronizeCache(CMDDRIVE *drive, BYTE immed);
 extern int SendReserveTrack(CMDDRIVE *drive, DWORD size);
 extern int SendRead10(CMDDRIVE *drive, DWORD lba, WORD len, DWORD buflen);
-extern int SendWrite10(CMDDRIVE *drive, DWORD lba, WORD len, DWORD buflen);
+extern int SendWrite10(CMDDRIVE *drive, DWORD lba, WORD len, DWORD buflen,
+					   BOOL retry);
 extern int SendSeek10(CMDDRIVE *drive, DWORD lba);
 extern int SendSendCueSheet(CMDDRIVE *drive, void *cuesheet, DWORD len);
+extern int SendReadBufferCapacity(CMDDRIVE *drive, BYTE block);
   
 extern int SendLongRead12(CMDDRIVE *drive, DWORD lba, DWORD len, DWORD buflen,
 						  DWORD total_blocks);
