@@ -55,17 +55,20 @@ CreateLog(CMDDRIVE *pReader, CMDDRIVE *pWriter, int nWriter)
   
 	for(nDrive=0; nDrive<1+nWriter; nDrive++){
 		if(nDrive==0){
-			fprintf(fp, "Ǎu : ");
-			pDrive = pReader;
+		  if(!REALDRIVE(pReader)){
+			continue;
+		  }
+		  fprintf(fp, "Ǎu : ");
+		  pDrive = pReader;
 		}
 		else{
-			fprintf(fp, "u%d : ", nDrive);
-			pDrive = &pWriter[nDrive-1];
+		  if(!REALDRIVE(&pWriter[nDrive-1])){
+			continue;
+		  }
+		  fprintf(fp, "u%d : ", nDrive);
+		  pDrive = &pWriter[nDrive-1];
 		}
 
-		if(pDrive->u.drive.hid < 0 && pDrive->u.drive.tid < 0){
-			continue;
-		}
 		ret = SendCmd(pDrive, (BYTE *)inqcmd, 96, REQ_DATAIN);
 		if(ret!=RET_OK){
 			fputs("\n", fp);
