@@ -1,12 +1,6 @@
 /* ~Օʉ
- * Copyright (C) 2004 Kagetani Hideto
+ * Copyright (C) 2004-2005 Kagetani Hideto
  */
-
-#ifdef linux
-# define _LARGEFILE_SOURCE
-# define _FILE_OFFSET_BITS 64
-#endif
-
 #include <stdio.h>
 #include <string.h>
 #include <stdarg.h>
@@ -16,6 +10,8 @@
 #include <time.h>
 #if defined(WIN32)
 # include <io.h>
+#else
+# include <unistd.h>
 #endif
 #if !defined(MACOSX)
 # include <malloc.h>
@@ -26,6 +22,7 @@
 #include "ui.h"
 #include "copydisc.h"
 #include "cmdlog.h"
+#include "netserver.h"
 
 
 typedef struct {
@@ -54,7 +51,6 @@ typedef struct {
 	CPTRACKINFO *trackinfo;	/* <= KŌ */
 } CPDISCINFO;
 
-#define REALDRIVE(d)	((d)->type==CMDDRVTYPE_DRIVE || (d)->type==CMDDRVTYPE_NET)
 
 
 static DWORD g_SpeedData[10];
@@ -66,12 +62,14 @@ static void InitSpeedData()
   g_NumSpeedData=0;
 }
 
-static void DispSpeed(DWORD lba, DWORD rest_blocks, CMDDRIVE *src)
+static void DispSpeed(DWORD lba, DWORD rest_blocks, CMDDRIVE *src, CMDDRIVE *dst)
 {
   double blocks_per_sec;
   double curr_speed;
   char info[80];
   DWORD rest_time;
+  int ret;
+  double buffer_capacity;
   
   if(g_NumSpeedData >= g_MaxSpeedData){
 	memmove(&g_SpeedData[0], &g_SpeedData[1],
@@ -96,8 +94,17 @@ static void DispSpeed(DWORD lba, DWORD rest_blocks, CMDDRIVE *src)
   sprintf(info, "%.1f{", curr_speed);
 	  /* c莞ԌvZ */
   rest_time = (DWORD)(0.5+rest_blocks/blocks_per_sec);
-  sprintf(info+strlen(info), " c莞 %d:%02d",
+  sprintf(info+strlen(info), " c莞=%d:%02d",
 		  (int)(rest_time/60), (int)(rest_time%60));
+
+  if(REALDRIVE(dst)){
+	ret = SendReadBufferCapacity(dst, 0);
+	if(ret == RET_OK){
+	  buffer_capacity = 100-
+		100.0*Get4bytes(dst->data_buf+8)/Get4bytes(dst->data_buf+4);
+	  sprintf(info+strlen(info), " ~=%.1f%%", buffer_capacity);
+	}
+  }
   UIDispInfo(info);
   if(src->type == CMDDRVTYPE_NET){
 	NAUIDispInfo(&src->u.net, info);
@@ -129,10 +136,6 @@ void DebugLog(char *fmt, ...)
 #endif
 }
 
-static int ReceiveNetCmd(CMDDRIVE *netdrive, CMDDRIVE *realdrive,
-						 BOOL enable_longread);
-
-
 
 static int OpenDevice(CMDDRIVE *drive, DRIVEID *id,
 					  BOOL reader, BOOL bWaitDisc,
@@ -156,11 +159,13 @@ static int OpenDevice(CMDDRIVE *drive, DRIVEID *id,
 	if(ret!=RET_ABORT && bWaitDisc){
 	  UIDispMessage("fBXN^Cv擾Ɏs܂B", UIDMT_ERROR);
 	}
+	FreeCmdDrive(drive);
 	return ret;
   }
   ret = SendPreventAllow(drive, 1);
   if(ret!=RET_OK){
 	DispCommandError(drive);
+	FreeCmdDrive(drive);
 	return ret;
   }
 
@@ -193,17 +198,21 @@ static int SetDiscInformation(CMDDRIVE *drive, CPDISCINFO *discinfo)
 		/* C[Wt@CɃfBXN */
 	discinfo->version = IMGFILE_VERSION;
 	ret = WriteImageFile(&drive->u.image, discinfo,
-						 sizeof(CPDISCINFO)-sizeof(CPTRACKINFO *));
+						 sizeof(CPDISCINFO)-sizeof(CPTRACKINFO *),
+						 TRUE);
 	if(ret!=RET_OK)
 	  return ret;
 	  
 	if(discinfo->cdtext_size>0){
-	  ret = WriteImageFile(&drive->u.image, discinfo->cdtext, discinfo->cdtext_size);
+	  ret = WriteImageFile(&drive->u.image,
+						   discinfo->cdtext, discinfo->cdtext_size,
+						   TRUE);
 	  if(ret!=RET_OK)
 		return ret;
 	}
 	ret = WriteImageFile(&drive->u.image, discinfo->trackinfo,
-						 sizeof(CPTRACKINFO)*discinfo->tracks);
+						 sizeof(CPTRACKINFO)*discinfo->tracks,
+						 TRUE);
 	if(ret!=RET_OK)
 	  return ret;
   }
@@ -525,7 +534,8 @@ static int GetDiscInformation(CMDDRIVE *drive, CPDISCINFO *discinfo)
 	else{
 		  /* C[Wt@CfBXN𓾂 */
 	  ret = ReadImageFile(&drive->u.image, discinfo,
-						  sizeof(CPDISCINFO)-sizeof(CPTRACKINFO *));
+						  sizeof(CPDISCINFO)-sizeof(CPTRACKINFO *),
+						  TRUE);
 	  if(ret!=RET_OK)
 		return ret;
 	  if(discinfo->version != IMGFILE_VERSION){
@@ -537,7 +547,8 @@ static int GetDiscInformation(CMDDRIVE *drive, CPDISCINFO *discinfo)
 		if(discinfo->cdtext==NULL)
 		  return RET_MEMERR;
 		ret = ReadImageFile(&drive->u.image, discinfo->cdtext,
-							discinfo->cdtext_size);
+							discinfo->cdtext_size,
+							TRUE);
 		if(ret!=RET_OK){
 		  free(discinfo->cdtext);
 		  discinfo->cdtext = NULL;
@@ -555,7 +566,8 @@ static int GetDiscInformation(CMDDRIVE *drive, CPDISCINFO *discinfo)
 		return RET_MEMERR;
 	  }
 	  ret = ReadImageFile(&drive->u.image, discinfo->trackinfo,
-						  sizeof(CPTRACKINFO)*discinfo->tracks);
+						  sizeof(CPTRACKINFO)*discinfo->tracks,
+						  TRUE);
 	  if(ret!=RET_OK){
 		free(discinfo->cdtext);
 		discinfo->cdtext = NULL;
@@ -1112,7 +1124,7 @@ static int GetVPSize(CMDDRIVE *drive, DWORD lba, DWORD *size_ret)
 	  return RET_OK;
 	}
 	else{
-	  ret = ReadImageFile(&drive->u.image, tmp, 4);
+	  ret = ReadImageFile(&drive->u.image, tmp, 4, TRUE);
 	  if(ret!=RET_OK)
 		return ret;
 	  *size_ret = Get4bytes(tmp);
@@ -1152,7 +1164,7 @@ static int SetVPSize(CMDDRIVE *drive, DWORD size)
 	}
 	else{
 	  Set4bytes(tmp, size);
-	  ret = WriteImageFile(&drive->u.image, tmp, 4);
+	  ret = WriteImageFile(&drive->u.image, tmp, 4, TRUE);
 	  if(ret!=RET_OK)
 		return ret;
 	}
@@ -1161,104 +1173,248 @@ static int SetVPSize(CMDDRIVE *drive, DWORD size)
   return RET_OK;
 }
 
+
+/**
+ * @brief	READgC񐔐ݒ(ModePage01h)
+ * @param[in]	drive	ΏۃhCu
+ * @param[in]	count	ݒ肷郊gC
+ * @param[out]	current	ݒÕgC
+ * @retval	RET_OK	I
+ * @retval	RET_CMDERR	R}hG[
+ * @retval	RET_NG	G[
+ */
+static int SetReadRetryCount(CMDDRIVE *drive, BYTE count, BYTE *current)
+{
+  int ret;
+  struct _MODEPAGE01 *mp01;
+  ret = SendModeSense(drive, MSPC_CURRENT, 1);
+  if(ret != RET_OK){
+	return ret;
+  }
+  mp01 = (struct _MODEPAGE01 *)
+	(drive->data_buf + (drive->cmd_type==CMDDRVCTYPE_ATAPI ? 8 : 16));
+  if(current != NULL){
+	*current = mp01->rd_retry_count;
+  }
+  mp01->rd_retry_count = count;
+  ret = SendModeSelect(drive, 1);
+  if(ret != RET_OK){
+	return ret;
+  }
+
+  return RET_OK;
+}
+
+/**
+ * @brief	READs
+ * @param[in]	src		ǂݍ݃hCu
+ * @param[in]	lba		JnLBA
+ * @param[in]	blocksize	ubNTCY
+ * @param[in]	len		ǂݍ݃ubN
+ * @param[in]	long_read	LONGREAD邩ǂ
+ * @param[in]	long_read_size	LONGREAD̃TCY
+ * @retval	RET_OK	I
+ * @retval	RET_CMDERR	R}hG[
+ * @retval	RET_NG	G[
+ */
+static int ExecRead(CMDDRIVE *src, DWORD lba, DWORD blocksize, DWORD len,
+					BOOL long_read, DWORD long_read_size)
+{
+  int ret;
+  BYTE *sp, *dp;
+  DWORD cnt;
+  
+	  /* Read */
+  if(!REALDRIVE(src)){
+	ret = ReadImageFile(&src->u.image, src->data_buf, blocksize*len,
+						(src->type==CMDDRVTYPE_IMAGE) ? TRUE:FALSE);
+	if(ret!=RET_OK)
+	  return ret;
+  }
+  else{
+	if(long_read){
+	  if(long_read_size){
+			/* LONGREADM */
+		if(blocksize==2048){
+		  ret = SendLongRead12(src, lba, (WORD)len, len*blocksize,
+							   long_read_size);
+		}
+		else{
+		  ret = SendLongReadCD(src, lba, len, long_read_size);
+		}
+		if(ret!=RET_OK){
+		  return ret;
+		}
+	  }
+		  /* LONGREADʎM */
+	  if(blocksize==2048){
+		ret = SendLongRead12(src, lba, (WORD)len, len*blocksize, 0);
+	  }
+	  else{
+		ret = SendLongReadCD(src, lba, len, 0);
+	  }
+	  if(ret!=RET_OK){
+		return ret;
+	  }
+	}
+	else{
+		  /* ʏREAD */
+	  if(blocksize==2048){
+		ret = SendRead10(src, lba, (WORD)len, len*blocksize);
+	  }
+	  else{
+		ret = SendReadCD(src, lba, len);
+	  }
+			
+	  if(ret!=RET_OK){
+		return ret;
+	  }
+	}
+	  
+	if(blocksize==2332){
+		  /* ReadCD bs=2352 œǂ񂾂ǂA
+			 KvȂ̂ 2332 Ȃ̂Ńobt@l߂ */
+	  sp = dp = src->data_buf;
+	  sp += 16;	/* sync 16bytes */
+	  for(cnt=0; cnt<len; cnt++){
+		memmove(dp, sp, 2332);
+		sp += 2352;
+		dp += 2332;
+	  }
+	}
+	else if(blocksize==2336){
+		  /* 2336 Ƀobt@l߂ */
+	  sp = dp = src->data_buf;
+	  sp += 16;	/* sync 16bytes */
+	  for(cnt=0; cnt<len; cnt++){
+		memmove(dp, sp, 2336);
+		sp += 2352;
+		dp += 2336;
+	  }
+	}
+  }
+
+  return RET_OK;
+}
+
+  
+
 /*
  * blocksize  2048/2332/2336/2352 삷B
  * ȊO͑ΉĂȂB
  */
 static int ExecReadWrite(CMDDRIVE *src, CMDDRIVE *dst, int num_dst,
 						 DWORD lba, DWORD blocksize, DWORD len,
-						 BOOL long_read, DWORD long_read_size)
+						 BOOL long_read, DWORD long_read_size,
+						 BOOL leadin)
 {
 	int ret;
-	BYTE *sp, *dp;
-	DWORD cnt;
 	int index;
+	int retry=0;
+	char info[80];
+	BYTE rdretry_keep=0;
 
-	if(!REALDRIVE(src) && !REALDRIVE(&dst[0])){
-		return RET_NG;
+	for(retry=0; retry<=32; retry++){
+	  if(retry > 0){
+		if(rdretry_keep == 0){
+			  /* hCuREADgC񐔂𑝂₷ */
+		  ret = SetReadRetryCount(src, 0xff, &rdretry_keep);
+		  if(ret != RET_OK){
+			rdretry_keep=0;
+		  }
+		}
+		if(retry==16){
+			  /* ăXsAbvĂ݂肷 */
+		  UIDispInfo("ĉ]");
+		  if(UICheckAbort()){
+			ret = RET_ABORT;
+			break;
+		  }
+		  SendStartStop(src, 1, 0, 0);
+#ifdef WIN32
+		  Sleep(1000);
+#else
+		  sleep(1);
+#endif
+		}
+		sprintf(info, "ēǍ:%d", retry);
+		UIDispInfo(info);
+		if(UICheckAbort()){
+		  ret = RET_ABORT;
+		  break;
+		}
+	  }
+	  ret = ExecRead(src, lba, blocksize, len, long_read, long_read_size);
+	  if(ret==RET_OK){
+		break;
+	  }
+	  else if(ret!=RET_CMDERR){
+		break;
+	  }
 	}
-
-	/* Read */
-	if(!REALDRIVE(src)){
-	  ret = ReadImageFile(&src->u.image, src->data_buf, blocksize*len);
-	  if(ret!=RET_OK)
-		return ret;
+	if(rdretry_keep > 0){
+		  /* hCuREADgC񐔐ݒ߂ */
+	  SetReadRetryCount(src, rdretry_keep, NULL);
 	}
-	else{
-	  if(long_read){
-		if(long_read_size){
-			  /* LONGREADM */
-		  if(blocksize==2048){
-			ret = SendLongRead12(src, lba, (WORD)len, len*blocksize,
-								 long_read_size);
-		  }
-		  else{
-			ret = SendLongReadCD(src, lba, len, long_read_size);
-		  }
-		  if(ret!=RET_OK){
-			return ret;
-		  }
-		}
-			/* LONGREADʎM */
-		if(blocksize==2048){
-		  ret = SendLongRead12(src, lba, (WORD)len, len*blocksize, 0);
-		}
-		else{
-		  ret = SendLongReadCD(src, lba, len, 0);
-		}
-		if(ret!=RET_OK){
-		  return ret;
-		}
-	  }
-	  else{
-		if(blocksize==2048){
-		  ret = SendRead10(src, lba, (WORD)len, len*blocksize);
-		}
-		else{
-		  ret = SendReadCD(src, lba, len);
-		}
-		if(ret!=RET_OK){
-		  DispCommandError(src);
-		  return ret;
-		}
-	  }
-	  
-	  if(blocksize==2332){
-			/* ReadCD bs=2352 œǂ񂾂ǂA
-			   KvȂ̂ 2332 Ȃ̂Ńobt@l߂ */
-		sp = dp = src->data_buf;
-		sp += 16;	/* sync 16bytes */
-		for(cnt=0; cnt<len; cnt++){
-		  memmove(dp, sp, 2332);
-		  sp += 2352;
-		  dp += 2332;
-		}
-	  }
-	  else if(blocksize==2336){
-			/* 2336 Ƀobt@l߂ */
-		sp = dp = src->data_buf;
-		sp += 16;	/* sync 16bytes */
-		for(cnt=0; cnt<len; cnt++){
-		  memmove(dp, sp, 2336);
-		  sp += 2352;
-		  dp += 2336;
-		}
-	  }
+	if(ret!=RET_OK){
+	  DispCommandError(src);
+	  return ret;
 	}
 
 	/* Write */
 	if(!REALDRIVE(&dst[0])){
-	  ret = WriteImageFile(&dst->u.image, dst->data_buf, len*blocksize);
+	  ret = WriteImageFile(&dst->u.image, dst->data_buf, len*blocksize,
+						   (dst->type==CMDDRVTYPE_IMAGE) ? TRUE:FALSE);
 	  if(ret!=RET_OK)
 		return ret;
 	}
 	else{
-	  for(index=0; index<num_dst; index++){
-		ret = SendWrite10(&dst[index], lba, (WORD)len, len*blocksize);
-		if(ret!=RET_OK){
-		  DispCommandError(&dst[index]);
-		  return ret;
+	  if(leadin && (num_dst>1)){
+			/*
+			 * SAOLead-inL^JnꍇAL^uœ
+			 * Lead-inL^Jn悤ɂB
+			 */
+		BOOL *started = (BOOL *)malloc(sizeof(BOOL)*num_dst);
+		int start_count=0;
+		if(started==NULL){
+		  return RET_MEMERR;
 		}
+		memset(started, 0, sizeof(BOOL)*num_dst);
+		while(start_count<num_dst){
+		  for(index=0; index<num_dst; index++){
+			if(started[index]){
+			  continue;
+			}
+			ret = SendWrite10(&dst[index], lba, (WORD)len, len*blocksize,
+							  FALSE);
+			if(ret==RET_CMDERR){
+			  if((SD_SENSEKEY(&dst[index])==0x02) &&
+				 (SD_ASC(&dst[index])==0x04) &&
+				 (SD_ASCQ(&dst[index])==0x08)){
+				continue;
+			  }
+			}
+			if(ret!=RET_OK){
+			  DispCommandError(&dst[index]);
+			  free(started);
+			  return ret;
+			}
+			started[index] = TRUE;
+			start_count++;
+		  }
+		}
+		free(started);
+	  }
+	  else{
+			/* Lead-inL^JnȂǂ͈ӎɕʂɋL^ */
+		for(index=0; index<num_dst; index++){
+		  ret = SendWrite10(&dst[index], lba, (WORD)len, len*blocksize, TRUE);
+		  if(ret!=RET_OK){
+			DispCommandError(&dst[index]);
+			return ret;
+		  }
+		}
+		
 	  }
 	}
 
@@ -1373,7 +1529,7 @@ static void CalcCDTextCRC(CPDISCINFO *discinfo)
 
 static int WriteCDText(CMDDRIVE *drive, int num_drv, CPDISCINFO *discinfo)
 {
-	int ret;
+	int ret = RET_OK;
 	struct _DISCINFO *di;
 	DWORD lba;
 	DWORD rest_blocks, total_blocks, len, deflen;
@@ -1382,52 +1538,90 @@ static int WriteCDText(CMDDRIVE *drive, int num_drv, CPDISCINFO *discinfo)
 	DWORD cdtext_offset=0;
 	long t1, t2;
 	DWORD i;
+	DWORD *leadin_start;
+	DWORD max_leadin;
+	int index;
 
 	if(!REALDRIVE(&drive[0]))
 		return RET_OK;
 
-	if(num_drv != 1){
-	  UIDispMessage("uւCD-TEXT݂͖T|[głB",
-					UIDMT_ERROR);
-	  return RET_NG;
+	leadin_start = (DWORD *)malloc(sizeof(DWORD)*num_drv);
+	if(leadin_start == NULL){
+	  return RET_MEMERR;
 	}
-	
+		/* Lead-inJnԂ擾 */
+	max_leadin = 0;
+	for(index=0; index<num_drv; index++){
+	  ret = SendReadDiscInfo(&drive[index]);
+	  if(ret!=RET_OK){
+		DispCommandError(&drive[index]);
+		free(leadin_start);
+		return ret;
+	  }
+	  di = (struct _DISCINFO *)drive[index].data_buf;
+	  leadin_start[index] = MSF2LBA(di->last_lead_in[1],
+									di->last_lead_in[2],
+									di->last_lead_in[3], FALSE);
+	  if(max_leadin < leadin_start[index]){
+		max_leadin = leadin_start[index];
+	  }
+	}
+		/*
+		 * }ĂfBXN̒ŁALead-inJnԂ
+		 * x̂ɍ킹B
+		 */
+	for(index=0; index<num_drv; index++){
+	  len = (drive[index].bufsize-1)/96;
+	  lba = leadin_start[index];
+	  rest_blocks = (DWORD)(max_leadin-lba);
+	  memset(drive[index].data_buf, 0, drive[index].bufsize);
+	  while(rest_blocks > 0){
+		if(len > rest_blocks){
+		  len = rest_blocks;
+		}
+		ret = SendWrite10(&drive[index], lba, (WORD)len, len*96, TRUE);
+		if(ret!=RET_OK){
+		  DispCommandError(&drive[index]);
+		  free(leadin_start);
+		  return ret;
+		}
+		rest_blocks -= len;
+		lba += len;
+	  }
+	}
+	free(leadin_start);
+
+		/* CD-TEXTf[^쐬 */
 	CalcCDTextCRC(discinfo);
 	converted_cdtext_size = (discinfo->cdtext_size * 4+2)/3;
 	converted_cdtext = malloc(converted_cdtext_size);
 	if(converted_cdtext==NULL)
-		return RET_MEMERR;
+	  return RET_MEMERR;
 	for(i=0; i<converted_cdtext_size/4; i++){
-		/*   8bit        6bit
-		 * aaaaaaaa    00aaaaaa
-		 * bbbbbbbb => 00aabbbb
-		 * cccccccc    00bbbbcc
-		 *             00cccccc
-		 */
-		converted_cdtext[i*4+0] = (discinfo->cdtext[i*3+0]>>2) & 0x3f;
-		converted_cdtext[i*4+1] = ((discinfo->cdtext[i*3+0]<<4) & 0x30) |
-								  ((discinfo->cdtext[i*3+1]>>4) & 0x0f);
-		converted_cdtext[i*4+2] = ((discinfo->cdtext[i*3+1]<<2) & 0x3c) |
-								  ((discinfo->cdtext[i*3+2]>>6) & 0x03);
-		converted_cdtext[i*4+3] = (discinfo->cdtext[i*3+2]>>0) & 0x3f;
+		  /*   8bit        6bit
+		   * aaaaaaaa    00aaaaaa
+		   * bbbbbbbb => 00aabbbb
+		   * cccccccc    00bbbbcc
+		   *             00cccccc
+		   */
+	  converted_cdtext[i*4+0] = (discinfo->cdtext[i*3+0]>>2) & 0x3f;
+	  converted_cdtext[i*4+1] = ((discinfo->cdtext[i*3+0]<<4) & 0x30) |
+		                        ((discinfo->cdtext[i*3+1]>>4) & 0x0f);
+	  converted_cdtext[i*4+2] = ((discinfo->cdtext[i*3+1]<<2) & 0x3c) |
+		                        ((discinfo->cdtext[i*3+2]>>6) & 0x03);
+	  converted_cdtext[i*4+3] = (discinfo->cdtext[i*3+2]>>0) & 0x3f;
 	}
 
-	/* Lead-inJnԂ擾 */
-	ret = SendReadDiscInfo(drive);
-	if(ret!=RET_OK){
-		DispCommandError(drive);
-		return ret;
-	}
-	di = (struct _DISCINFO *)drive->data_buf;
-	lba = MSF2LBA(di->last_lead_in[1],
-					di->last_lead_in[2],
-					di->last_lead_in[3], FALSE);
+	lba = max_leadin;
 	rest_blocks = total_blocks = (DWORD)(-150-lba);
 
-	/* 1xɓ]łubNvZ(1ubN=96bytes) */
+		/*
+		 * 1xɓ]łubNvZ(1ubN=96bytes)
+		 * AAShCũobt@TCYƂB
+		 */
 	deflen = (drive->bufsize-1)/96;
 
-	/* ʕ\ */
+		/* ʕ\ */
 	UIMeter2Initialize("CD-TEXT");
 	if(drive->type==CMDDRVTYPE_NET){
 	  NAUIMeter2Initialize(&drive->u.net, "CD-TEXT");
@@ -1435,31 +1629,36 @@ static int WriteCDText(CMDDRIVE *drive, int num_drv, CPDISCINFO *discinfo)
 	t1 = t2 = time(NULL);
 
 	while(rest_blocks>0){
-		t2 = time(NULL);
-		if(t1!=t2){
-			UIMeter2Update((total_blocks-rest_blocks)*100/total_blocks);
-			if(drive->type==CMDDRVTYPE_NET){
-			  NAUIMeter2Update(&drive->u.net,
-							   (BYTE)((total_blocks-rest_blocks)
-									  *100/total_blocks));
-			}
-			t1=t2;
-		}
-		if(UICheckAbort()){
-			ret = RET_ABORT;
-			break;
+	  t2 = time(NULL);
+	  if(t1!=t2){
+		UIMeter2Update((total_blocks-rest_blocks)*100/total_blocks);
+		if(drive->type==CMDDRVTYPE_NET){
+		  NAUIMeter2Update(&drive->u.net,
+						   (BYTE)((total_blocks-rest_blocks)
+								  *100/total_blocks));
 		}
+		t1=t2;
+	  }
+	  if(UICheckAbort()){
+		ret = RET_ABORT;
+		break;
+	  }
 
-		len = deflen < rest_blocks ? deflen : rest_blocks;
-		MakeCDTextBuffer(drive->data_buf, len*96,
-						converted_cdtext, converted_cdtext_size, &cdtext_offset);
-		ret = SendWrite10(drive, lba, (WORD)len, len*96);
+	  len = deflen < rest_blocks ? deflen : rest_blocks;
+	  for(index=0; index<num_drv; index++){
+		MakeCDTextBuffer(drive[index].data_buf, len*96,
+						 converted_cdtext, converted_cdtext_size, &cdtext_offset);
+		ret = SendWrite10(&drive[index], lba, (WORD)len, len*96, TRUE);
 		if(ret!=RET_OK){
-			DispCommandError(drive);
-			break;
+		  DispCommandError(&drive[index]);
+		  break;
 		}
-		lba += len;
-		rest_blocks -= len;
+	  }
+	  if(ret!=RET_OK){
+		break;
+	  }
+	  lba += len;
+	  rest_blocks -= len;
 	}
 
 	free(converted_cdtext);
@@ -1470,7 +1669,8 @@ static int WriteCDText(CMDDRIVE *drive, int num_drv, CPDISCINFO *discinfo)
 
 static int WriteLoop(CMDDRIVE *src, CMDDRIVE *dst, int num_dst,
 					 DWORD lba, DWORD blocks, DWORD deflen,
-					 DWORD blocksize, BOOL vp_track, DWORD last_addr, BOOL sao)
+					 DWORD blocksize, BOOL vp_track, DWORD last_addr,
+					 BOOL sao, BOOL leadin)
 {
 	int ret;
 	DWORD rest_blocks, total_blocks, len;
@@ -1523,7 +1723,7 @@ static int WriteLoop(CMDDRIVE *src, CMDDRIVE *dst, int num_dst,
 							 (BYTE)((total_blocks-rest_blocks)*100/total_blocks));
 		  }
 			  /* x,c莞Ԃ\ */
-		  DispSpeed(lba, rest_blocks, src);
+		  DispSpeed(lba, rest_blocks, src, dst);
 		  t1=t2;
 		}
 		if(UICheckAbort()){
@@ -1543,7 +1743,8 @@ static int WriteLoop(CMDDRIVE *src, CMDDRIVE *dst, int num_dst,
 		  }
 		}
 		ret = ExecReadWrite(src, dst, num_dst, lba, blocksize, len,
-							long_read, (long_read_now ? long_read_size : 0));
+							long_read, (long_read_now ? long_read_size : 0),
+							leadin);
 		if(ret!=RET_OK)
 		  break;
 		lba += len;
@@ -1670,7 +1871,7 @@ static int CopyTrack(CMDDRIVE *src, CMDDRIVE *dst, int num_dst,
 	}
 
 	ret = WriteLoop(src, dst, num_dst, lba, blocks, len, blocksize,
-					vp_track, discinfo->last_addr, FALSE);
+					vp_track, discinfo->last_addr, FALSE, FALSE);
 	if(ret!=RET_OK){
 		  /* G[̏ꍇ́ASyncCacheďI */
 	  for(index=0; index<num_dst; index++){
@@ -1931,6 +2132,7 @@ static int CopySession(CMDDRIVE *src, CMDDRIVE *dst, int num_dst,
 	DWORD translen, blocksize;
 	char msgbuf[80];
 	int index;
+	BOOL leadin=FALSE;
 
 	for(index=0; index<num_dst; index++){
 	  if(cs!=NULL){
@@ -1958,6 +2160,7 @@ static int CopySession(CMDDRIVE *src, CMDDRIVE *dst, int num_dst,
 		if(csp->dataform==0x01){
 		  if(REALDRIVE(dst)){
 			UIDispInfo("L^");
+			leadin = TRUE;
 		  }
 		}
 		continue;
@@ -2006,11 +2209,12 @@ static int CopySession(CMDDRIVE *src, CMDDRIVE *dst, int num_dst,
 
 	  ret = WriteLoop(src, dst, num_dst,
 					  lba, blocks, translen, blocksize,
-					  FALSE, discinfo->last_addr, TRUE);
+					  FALSE, discinfo->last_addr, TRUE, leadin);
 	  if(ret!=RET_OK){
 		break;
 	  }
 	  *track_num_ret = csp->tno;
+	  leadin = FALSE;
 	}
 
 	for(index=0; index<num_dst; index++){
@@ -2389,11 +2593,13 @@ static int OpenNetDevice(CMDDRIVE *drive, BOOL server,
 	  if(ret!=RET_ABORT){
 		UIDispMessage("fBXN^Cv擾Ɏs܂B", UIDMT_ERROR);
 	  }
+	  FreeCmdDrive(drive);
 	  return ret;
 	}
 	ret = SendPreventAllow(drive, 1);
 	if(ret!=RET_OK){
 	  DispCommandError(drive);
+	  FreeCmdDrive(drive);
 	  return ret;
 	}
   }
@@ -2457,285 +2663,7 @@ void PostProcess(int retcode, CMDDRIVE *src, CMDDRIVE *dstp, int num_dst)
   }
 }
 
-static int ExecuteLongRead(CMDDRIVE *netdrive, CMDDRIVE *realdrive,
-						   NETCMDHEADER *org_header,
-						   DWORD start_lba, DWORD total_length)
-{
-  DWORD lba;
-  DWORD length;
-  DWORD blocksize;
-  int ret;
-  BYTE netdata_common[8];
-  NETCMDHEADER header;
 
-  memcpy(header.cdb, org_header->cdb, 12);
-  lba = start_lba;
-  if(header.cdb[0]==CMD_READ12){
-	length = Get4bytes(header.cdb+6);
-	blocksize = 2048;
-  }
-  else if(header.cdb[0]==CMD_READ_CD){
-	length = Get3bytes(header.cdb+6);
-	blocksize = 2352;
-  }
-  else{
-	return RET_NG;
-  }
-
-  while(total_length > 0){
-	if(UICheckAbort()){
-	  return RET_ABORT;
-	}
-	if(length > total_length){
-	  length = total_length;
-	  if(header.cdb[0]==CMD_READ12){
-		Set4bytes(header.cdb+2, lba);
-		Set4bytes(header.cdb+6, length);
-	  }
-	  else /* CMD_READ_CD */{
-		Set4bytes(header.cdb+2, lba);
-		Set3bytes(header.cdb+6, length);
-	  }
-	}
-		/* hCuREAD */
-	Set4bytes(header.cdb+2, lba);
-	ret = SendCmd(realdrive, header.cdb, length*blocksize, REQ_DATAIN);
-	if(ret != RET_OK){
-	  return ret;
-	}
-	memcpy(header.sense_data, realdrive->sense_data,
-		   sizeof(header.sense_data));
-
-		/* READf[^ԑ */
-	Set4bytes(netdata_common, length*blocksize+sizeof(header));
-	netdata_common[4] = NETDATATYPE_RESPONSE;
-	while(1){
-	  ret = NASend(&netdrive->u.net, (char *)netdata_common, 8,
-				   NETTIMEOUT, TRUE);
-	  if(ret==RET_OK){
-		break;
-	  }
-	  if(ret!=RET_READFD){
-		return ret;
-	  }
-	  ret = ReceiveNetCmd(netdrive, realdrive, FALSE);
-	  if(ret != RET_OK){
-		return ret;
-	  }
-	}
-	ret = NASend(&netdrive->u.net, (char *)&header, sizeof(header),
-				 NETTIMEOUT, FALSE);
-	if(ret != RET_OK){
-	  return ret;
-	}
-	ret = NASend(&netdrive->u.net, (char *)netdrive->data_buf,
-				 length*blocksize, NETTIMEOUT, FALSE);
-	if(ret != RET_OK){
-	  return ret;
-	}
-
-	lba += length;
-	total_length -= length;
-  }
-
-  return RET_OK;
-}
-
-
-static int ReceiveNetCmd(CMDDRIVE *netdrive, CMDDRIVE *realdrive,
-						 BOOL enable_longread)
-{
-  int ret;
-  NETCMDHEADER cmdheader;
-  NETDISPHEADER dispheader;
-  BYTE netdata_common[8];
-  DWORD length;
-  DWORD receive_bytes;
-  BYTE work4[4];
-  char *message=NULL;
-  
-	  /* ʕ(f[^ƃf[^^Cv)M */
-  ret = NAReceive(&netdrive->u.net, (char *)netdata_common, 8, 1000);
-  if(ret != RET_OK){
-	return ret;
-  }
-
-  receive_bytes = Get4bytes(netdata_common);
-
-  switch(netdata_common[4]){
-  case NETDATATYPE_COMMAND:
-		/* R}hM */
-	ret = NAReceive(&netdrive->u.net, (char *)&cmdheader, sizeof(cmdheader),
-					NETTIMEOUT);
-	if(ret != RET_OK){
-	  return ret;
-	}
-	length = Get4bytes(cmdheader.data_length);
-	if(length > (DWORD)netdrive->bufsize){
-	  length = (DWORD)netdrive->bufsize;
-	}
-	
-	if((length > 0) && (cmdheader.reqflag==REQ_DATAOUT)){
-		  /* R}hf[^M */
-	  ret = NAReceive(&netdrive->u.net, (char *)netdrive->data_buf,
-					  (int)length, NETTIMEOUT);
-	  if(ret != RET_OK){
-		return ret;
-	  }
-	}
-	ret = SendCmd(realdrive, cmdheader.cdb, length, cmdheader.reqflag);
-		/* G[ĂAG[͕ԋp */
-		/* f[^ԑ */
-	if(cmdheader.reqflag == REQ_DATAIN){
-	  Set4bytes(netdata_common, length+sizeof(NETCMDHEADER));
-	}
-	else{
-	  Set4bytes(netdata_common, sizeof(NETCMDHEADER));
-	}
-	netdata_common[4] = NETDATATYPE_RESPONSE;
-	ret = NASend(&netdrive->u.net, (char *)netdata_common, 8,
-				 NETTIMEOUT, FALSE);
-	if(ret != RET_OK){
-	  return ret;
-	}
-	memcpy(cmdheader.sense_data, realdrive->sense_data,
-		   sizeof(cmdheader.sense_data));
-	ret = NASend(&netdrive->u.net, (char *)&cmdheader, sizeof(cmdheader),
-				 NETTIMEOUT, FALSE);
-	if(ret != RET_OK){
-	  return ret;
-	}
-	if(cmdheader.reqflag == REQ_DATAIN){
-	  ret = NASend(&netdrive->u.net, (char *)netdrive->data_buf, length,
-				   NETTIMEOUT, FALSE);
-	  if(ret != RET_OK){
-		return ret;
-	  }
-	}
-	
-	break;
-	
-  case NETDATATYPE_RESPONSE:
-		/* 蓾Ȃ */
-	break;
-	
-  case NETDATATYPE_LONGREAD:
-	if(enable_longread){
-		  /* R}hM */
-	  ret = NAReceive(&netdrive->u.net, (char *)&cmdheader, sizeof(cmdheader),
-					  NETTIMEOUT);
-	  if(ret != RET_OK){
-		return ret;
-	  }
-		  /* vubNM */
-	  ret = NAReceive(&netdrive->u.net, (char *)work4, sizeof(work4),
-					  NETTIMEOUT);
-	  if(ret != RET_OK){
-		return ret;
-	  }
-	  ret = ExecuteLongRead(netdrive, realdrive,
-							&cmdheader,
-							Get4bytes(cmdheader.cdb+2)	/* LBA */,
-							Get4bytes(work4)	/* total length */);
-	  if(ret != RET_OK){
-		return ret;
-	  }
-	}
-	else{
-	  return RET_NG;
-	}
-	break;
-	
-  case NETDATATYPE_DISPLAY:
-	ret = NAReceive(&netdrive->u.net, (char *)&dispheader, sizeof(dispheader),
-					NETTIMEOUT);
-	if(ret != RET_OK){
-	  return ret;
-	}
-	if(receive_bytes > (DWORD)sizeof(dispheader)){
-		  /* bZ[WM */
-	  message = (char *)malloc(receive_bytes-sizeof(dispheader)+1);
-	  if(message==NULL){
-		UIDispMessage("mۂɎs܂B", UIDMT_ERROR);
-		return RET_MEMERR;
-	  }
-	  ret = NAReceive(&netdrive->u.net, (char *)message,
-					  receive_bytes-sizeof(dispheader), NETTIMEOUT);
-	  if(ret != RET_OK){
-		free(message);
-		return ret;
-	  }
-	  message[receive_bytes-sizeof(dispheader)] = '\0';
-#if defined(linux)
-	  {
-		char *message_euc = ConvertCharset(netdrive->u.net.iconv_desc,
-										   message);
-		if(message_euc == NULL){
-		  free(message);
-		  UIDispMessage("R[hϊɎs܂B", UIDMT_ERROR);
-		  return RET_MEMERR;
-		}
-		free(message);
-		message = message_euc;
-	  }
-#endif
-	}
-	switch(dispheader.area){
-	case 0:	/* ʉbZ[W */
-	  UIDispInfo(message);
-	  break;
-	case 1:	/* vOXo[1 */
-	  if(message!=NULL)
-		UIMeter1Initialize(message);
-	  else
-		UIMeter1Update(dispheader.percent);
-	  break;
-	case 2:	/* vOXo[2 */
-	  if(message!=NULL)
-		UIMeter2Initialize(message);
-	  else
-		UIMeter2Update(dispheader.percent);
-	  break;
-	}
-	free(message);
-	message=NULL;
-	break;
-	
-  case NETDATATYPE_ABORT:
-	return RET_ABORT;
-
-  case NETDATATYPE_COMPLETE:
-	return RET_COMPLETE;
-
-  default:
-		/* sȃR}h */
-	UIDispMessage("sȃR}hM܂B", UIDMT_ERROR);
-	return RET_NG;
-  }
-  return RET_OK;
-}
-
-
-int WaitingNetCmd(CMDDRIVE *netdrive, CMDDRIVE *realdrive)
-{
-  BOOL done=FALSE;
-  int ret;
-  
-  while(!done){
-	if(UICheckAbort()){
-	  return RET_ABORT;
-	}
-	ret = ReceiveNetCmd(netdrive, realdrive, TRUE);
-	if(ret == RET_COMPLETE){
-	  done=TRUE;
-	}
-	else if((ret != RET_OK) && (ret != RET_TIMEOUT)){
-	  return ret;
-	}
-  }
-  
-  return RET_OK;
-}
 
 
 int CopyDisc(DRIVEID *reader, DRIVEID *writer, int num_writer)
@@ -2756,6 +2684,8 @@ int CopyDisc(DRIVEID *reader, DRIVEID *writer, int num_writer)
 	memset(&dst, 0, sizeof(dst));
 	memset(&virtual_drv, 0, sizeof(virtual_drv));
 
+	UICheckAbort();
+	
 	dstp = (CMDDRIVE *)malloc(num_writer*sizeof(CMDDRIVE));
 	if(dstp==NULL){
 	  UIDispMessage("mۂɎs܂B", UIDMT_ERROR);
@@ -2922,7 +2852,11 @@ int CopyDisc(DRIVEID *reader, DRIVEID *writer, int num_writer)
 		  /* On-The-Fly */
 	  bDVD = DT_DVD_FAMILY(disc_type);
 	  for(index=0; index<num_writable_drive; index++){
-		if(bDVD != DT_DVD_FAMILY(dstp[index].disc_type)){
+		if(!REALDRIVE(&dstp[index])){
+		  dstp[index].disc_type = disc_type;
+		}
+		else if((bDVD != DT_DVD_FAMILY(dstp[index].disc_type)) &&
+				REALDRIVE(&src)){
 		  UIDispMessage("CD  DVD ADVD  CD ɂ̓Rs[ł܂B",
 						UIDMT_ERROR);
 		  CloseDevice(&src);
@@ -3001,8 +2935,8 @@ int CopyDisc(DRIVEID *reader, DRIVEID *writer, int num_writer)
 	    ret = OpenDevice(&dstp[0], writer, FALSE, TRUE, data_buf, bufsize);
 	    if(ret!=RET_OK){
 		  PostProcess(ret, &src, dstp, num_writable_drive);
-		  RemoveImageFile(tmpfile);
 	      CloseDevice(&virtual_drv);
+		  RemoveImageFile(tmpfile);
 		  if(!bSameDrive){
 			for(index=0; index<num_writable_drive; index++){
 			  CloseDevice(&dstp[index]);
@@ -3017,8 +2951,8 @@ int CopyDisc(DRIVEID *reader, DRIVEID *writer, int num_writer)
 		  /* Image file => DISC */
 	  ret = ExecCopy(&virtual_drv, dstp, num_writable_drive);
 	  PostProcess(ret, &src, dstp, num_writable_drive);
-	  RemoveImageFile(tmpfile);
 	  CloseDevice(&virtual_drv);
+	  RemoveImageFile(tmpfile);
 	  if(!bSameDrive){
 		for(index=0; index<num_writable_drive; index++){
 		  CloseDevice(&dstp[index]);
