@@ -5,12 +5,20 @@
 #define __COPYDISC_H__
 
 #include "aspi.h"
+#include "cmd.h"
+
+#define HID_VIRTUAL		100
+
+typedef struct {
+  int hid;
+  int tid;
+} DRIVEID;
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-int CopyDisc(DRIVE *reader, DRIVE *writer, int num_writer);
+int CopyDisc(DRIVEID *reader, DRIVEID *writer, int num_writer);
 
 #ifdef __cplusplus
 }
