@@ -17,8 +17,8 @@ typedef struct {
 extern int InitializeImage(IMAGEFILE *image);
 extern void FreeImage(IMAGEFILE *image);
 extern int OpenImage(IMAGEFILE *image, const char *filename, BOOL reader);
-extern int ReadImageFile(IMAGEFILE *image, void *buf, DWORD size);
-extern int WriteImageFile(IMAGEFILE *image, void *buf, DWORD size);
+extern int ReadImageFile(IMAGEFILE *image, void *buf, DWORD size, BOOL split);
+extern int WriteImageFile(IMAGEFILE *image, void *buf, DWORD size, BOOL split);
 extern void RemoveImageFile(const char *basefilename);
 
 #endif /* !__IMAGE_H__ */
