@@ -2,6 +2,9 @@
  * Copyright (C) 2004 Kagetani Hideto
  */
 #if !defined(WIN32)
+# if defined(__APPLE__) && defined(__MACH__)
+#  include <sys/time.h>
+# endif
 # include <errno.h>
 # include <stdlib.h>
 # include <string.h>
@@ -30,6 +33,17 @@ static void SetErrorCode(SOCKCB *scb)
 #endif
 }
 
+/**
+ * @brief	\Pbg҂
+ * @param[out]	read_fds	ǂݍݗp
+ * @param[out]	write_fds	ݗp
+ * @param[out]	except_fds	Op
+ * @param[in]	sd			\PbgfXNv^
+ * @param[in]	timeout		^CAEg(~b)
+ * @retval	RET_OK	\Pbgo͗L
+ * @retval	RET_TIMEOUT	^CAEg
+ * @retval	RET_NG	G[
+ */
 static int WaitFDS(fd_set *read_fds, fd_set *write_fds, fd_set *except_fds,
 					SOCKET sd, DWORD timeout)
 {
@@ -244,6 +258,8 @@ int NAClose(SOCKCB *scb)
   SOCKET sd;
   int read_bytes;
   char buf[256];
+  fd_set read_fds;
+  int ret;
 
   if(scb->server_mode){
 	sd = scb->u.server.socket_desc;
@@ -256,6 +272,13 @@ int NAClose(SOCKCB *scb)
 	shutdown(sd, SD_SEND);
 
 	while(1){
+	  ret = WaitFDS(&read_fds, NULL, NULL, sd, 30*1000);
+	  if(ret == RET_TIMEOUT){
+		continue;
+	  }
+	  else if(ret != RET_OK){
+		break;
+	  }
 	  read_bytes = recv(sd, buf, sizeof(buf), 0);
 	  if(read_bytes == SOCKET_ERROR){
 		break;
@@ -330,12 +353,25 @@ int NASend(SOCKCB *scb, const char *buffer, int length, DWORD timeout,
   return RET_OK;
 }
 
+/**
+ * @brief	\Pbgf[^M
+ * @param[in]	scb		\PbgubN
+ * @param[out]	buffer	Mf[^i[̈(NULLwŃf[^̂Ă)
+ * @param[in]	length	Mf[^
+ * @param[in]	timeout	^CAEg(~b)
+ * @retval	RET_OK	I
+ * @retval	RET_TIMEOUT	^CAEg
+ * @retval	RET_ABORT	f(葤ؒfꂽ)
+ * @retval	RET_SOCKET	\PbgG[
+ */
 int NAReceive(SOCKCB *scb, char *buffer, int length, DWORD timeout)
 {
   fd_set read_fds;
   int ret;
   SOCKET sd;
   int read_bytes;
+  char dummy_buffer[64];
+  int len;
 
   if(scb->server_mode){
 	sd = scb->u.server.socket_desc;
@@ -355,7 +391,13 @@ int NAReceive(SOCKCB *scb, char *buffer, int length, DWORD timeout)
 	}
 
 	if(FD_ISSET(sd, &read_fds)){
-	  read_bytes = recv(sd, buffer, length, 0);
+	  if(buffer==NULL){
+		len = (length < sizeof(buffer)) ? length : sizeof(buffer);
+		read_bytes = recv(sd, dummy_buffer, len, 0);
+	  }
+	  else{
+		read_bytes = recv(sd, buffer, length, 0);
+	  }
 	  if(read_bytes == SOCKET_ERROR){
 		SetErrorCode(scb);
 		return RET_SOCKET;
@@ -539,6 +581,7 @@ char *ConvertCharset(iconv_t iconv_desc, const char *in_string)
   char *inbufp = (char *)in_string;
   char *outbufp;
   int out_offset = 0;
+  int iconv_ret;
 
   out_string = (char *)malloc(bufsize);
   
@@ -546,15 +589,14 @@ char *ConvertCharset(iconv_t iconv_desc, const char *in_string)
 	out_string[0] = '\0';
 	return out_string;
   }
-  
+
+  outbufp = out_string;
   while(inbytes > 0){
-	outbufp = out_string+out_offset;
-	if(iconv(iconv_desc,
-			 &inbufp, &inbytes,
-			 &outbufp, &outbytes) >= 0){
-		  /* ϊ */
-	  *outbufp = '\0';
-	  break;
+	iconv_ret = iconv(iconv_desc,
+					  &inbufp, &inbytes,
+					  &outbufp, &outbytes);
+	if(iconv_ret >= 0){
+	  continue;
 	}
 	if(errno != E2BIG){
 		  /* ϊs */
@@ -564,11 +606,14 @@ char *ConvertCharset(iconv_t iconv_desc, const char *in_string)
 		/* ̈s */
 	out_offset = (int)(outbufp-out_string);
 	bufsize += 64;
+	outbytes += 64;
 	out_string = (char *)realloc(out_string, bufsize);
 	if(out_string == NULL){
 	  return NULL;
 	}
+	outbufp = out_string+out_offset;
   }
+  *outbufp = '\0';
   
   return out_string;
 }
