@@ -221,25 +221,25 @@ int SetOption(CMDDRIVE *reader, CMDDRIVE *writer, int disc_type)
 		g_Option.test_write = FALSE;
 	}
 	else{
-		if(writer==NULL){
-			g_Option.bufe = FALSE;
-			g_Option.test_write = FALSE;
-		}
-		else{
-			ret = SendModeSense(writer, MSPC_CHANGEABLE, 5);
-			if(ret!=RET_OK){
-				DispCommandError(writer);
-				return ret;
-			}
-			mp05 = (struct _MODEPAGE05 *)
-				(writer->data_buf + (writer->cmd_type==CMDDRVCTYPE_ATAPI ? 8:16));
-			if(mp05->BUFE){
-				g_Option.bufe = TRUE;
-				g_Option.flags |= OPFLG_BUFE;
-			}
-			if(mp05->test_write){
-				g_Option.test_write = FALSE;
-				g_Option.flags |= OPFLG_TESTWRITE;
+		g_Option.bufe = FALSE;
+		g_Option.test_write = FALSE;
+		if(writer!=NULL){
+			if((writer->type == CMDDRVTYPE_DRIVE) || (writer->type == CMDDRVTYPE_NET)){
+				ret = SendModeSense(writer, MSPC_CHANGEABLE, 5);
+				if(ret!=RET_OK){
+					DispCommandError(writer);
+					return ret;
+				}
+				mp05 = (struct _MODEPAGE05 *)
+					(writer->data_buf + (writer->cmd_type==CMDDRVCTYPE_ATAPI ? 8:16));
+				if(mp05->BUFE){
+					g_Option.bufe = TRUE;
+					g_Option.flags |= OPFLG_BUFE;
+				}
+				if(mp05->test_write){
+					g_Option.test_write = FALSE;
+					g_Option.flags |= OPFLG_TESTWRITE;
+				}
 			}
 		}
 	}
