@@ -14,7 +14,7 @@
 
 extern void DebugLog(char *fmt, ...);
 
-
+#if !defined(MACOSX)
 static BOOL IsWriterDrive(CMDDRIVE *drive)
 {
 	int ret;
@@ -31,18 +31,21 @@ static BOOL IsWriterDrive(CMDDRIVE *drive)
 	/* GetConfig.Ȃƃ_ */
 	return FALSE;
 }
+#endif
 
 
 SCAN_t *ScanDrive(BOOL bWriter)
 {
+#if !defined(MACOSX)
 	const BYTE inqcmd[12]={CMD_INQUIRY, 0, 0, 0, 96, 0, 0, 0, 0, 0, 0, 0};
 	DWORD timeout=0;
 	BYTE device_type;
+	int ret;	
+#endif
 	int num_drive=0;
 	int hid, tid, max_hid, max_tid;
 	SCAN_t *drives;
 	CMDDRIVE drive;
-	int ret;
 
 	drives = (SCAN_t *)malloc(sizeof(SCAN_t)*MAX_SCAN);
 
@@ -64,7 +67,7 @@ SCAN_t *ScanDrive(BOOL bWriter)
 				break;
 			}
 			DebugLog("[%d:%d]\n", hid, tid);
-#if 0 /*def MACOSX*/
+#ifdef MACOSX
 			if(GetInquiryString(tid, drive.data_buf, drive.bufsize)){
 				continue;
 			}
@@ -116,10 +119,15 @@ SCAN_t *ScanDrive(BOOL bWriter)
 			drives[num_drive].hid = hid;
 			drives[num_drive].tid = tid;
 			if(bWriter){
-				if(IsWriterDrive(&drive)==FALSE){
-				  printf("not a writer\n");
+#ifdef MACOSX
+				if(MMCIsWriterDrive(tid, drive.data_buf, drive.bufsize)==FALSE){
 					continue;
 				}
+#else
+				if(IsWriterDrive(&drive)==FALSE){
+					continue;
+				}
+#endif
 			}
 			num_drive++;
 		}
