@@ -28,6 +28,8 @@ static SCAN_t *g_WriterList=NULL;
 static int g_AllWritableDrives= -1;
 static int g_ReaderISO = -1;
 static int g_WriterISO = -1;
+static int g_ReaderNet = -1;
+static int g_WriterNet = -1;
 
 
 static int scan_drive(GtkWidget *optmenu, BOOL writer)
@@ -73,6 +75,17 @@ static int scan_drive(GtkWidget *optmenu, BOOL writer)
   }
   count++;
 
+  menuitem = gtk_menu_item_new_with_label("ͥåȥ");
+  gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(count));
+  gtk_menu_append(GTK_MENU(menu), menuitem);
+  gtk_widget_show(menuitem);
+  if(writer){
+    g_WriterNet = count;
+  }
+  else{
+    g_ReaderNet = count;
+  }
+  count++;
 
   gtk_option_menu_set_menu(GTK_OPTION_MENU(optmenu), menu);
 
@@ -102,15 +115,12 @@ on_optionmenu2_map(GtkWidget *widget, gpointer user_data)
 void
 on_btnStartCopy_clicked(GtkWidget *widget, gpointer user_data)
 {
-  DRIVE reader, writer, *writerp=NULL;
+  DRIVEID reader, *writerp=NULL;
   GtkWidget *menu;
   GtkWidget *menuitem;
   int sel_reader, sel_writer;
-  DWORD timeout = 5*60;	/* ޥɥॢ=5ʬ */
   int ret;
-  gint16 mbret;
   int writer_num=1, index;
-  BOOL init_error = FALSE;
 
   if(g_bRunningCopy){
     UISetAbort();
@@ -118,105 +128,75 @@ on_btnStartCopy_clicked(GtkWidget *widget, gpointer user_data)
     return;
   }
 
-  memset(&reader, 0x00, sizeof(reader));
-  memset(&writer, 0x00, sizeof(writer));
-
 	  /* ɥ饤ּ */
   menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(g_optMenu1));
   menuitem = gtk_menu_get_active(GTK_MENU(menu));
   sel_reader = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
-
+  
   menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(g_optMenu2));
   menuitem = gtk_menu_get_active(GTK_MENU(menu));
   sel_writer = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
 
-	  /* ɥ饤ֹ¤ν */
-  memset(&reader, 0, sizeof(DRIVE));
-  memset(&writer, 0, sizeof(DRIVE));
-  ret = InitializeDrive(&reader, 0x10000, 50);
-  if(ret==RET_OK){
-    if(sel_reader != g_ReaderISO){
-      ret = SetAspiSetting(&reader,
-			   &g_ReaderList[sel_reader].hid,
-			   &g_ReaderList[sel_reader].tid,
-			   &timeout);
-    }
-  }
-  if(ret!=RET_OK){
-    MessageBoxDlg("ɹ֤ν˼Ԥޤ", NULL,
-		  MBF_TYPEOK|MBF_ICONSTOP);
-    return;
+	  /* ѥ */
+  writerp = (DRIVEID *)malloc(sizeof(DRIVEID));
+  if(writerp==NULL){
+	MessageBoxDlg("֤ν˼Ԥޤ(ݥ顼)", NULL,
+				  MBF_TYPEOK|MBF_ICONSTOP);
+	return;
   }
+
+	  /*  */
   if(sel_writer == g_AllWritableDrives){
     writer_num = 0;
     for(index=0; strlen(g_WriterList[index].inqdata)!=0; index++){
       if(g_ReaderList[sel_reader].hid == g_WriterList[index].hid &&
-	 g_ReaderList[sel_reader].tid == g_WriterList[index].tid){
-	/* ɼ̵֤ */
-	continue;
+		 g_ReaderList[sel_reader].tid == g_WriterList[index].tid){
+			/* ɼ̵֤ */
+		continue;
       }
-      writerp = (DRIVE *)realloc(writerp, (writer_num+1)*sizeof(DRIVE));
+      writerp = (DRIVEID *)realloc(writerp, (writer_num+1)*sizeof(DRIVEID));
       if(writerp == NULL){
-	init_error = TRUE;
-	break;
-      }
-      memset(&writerp[writer_num], 0, sizeof(DRIVE));
-      ret = InitializeDrive(&writerp[writer_num], 0, 50);
-      if(ret!=RET_OK){
-	init_error = TRUE;
-	break;
-      }
-      /* writerΥХåեreaderΥХåեͭ */
-      writerp[writer_num].data_buf = reader.data_buf;
-      writerp[writer_num].bufsize = reader.bufsize;
-      ret = SetAspiSetting(&writerp[writer_num],
-			   &g_WriterList[index].hid,
-			   &g_WriterList[index].tid,
-			   &timeout);
-      if(ret!=RET_OK){
-	init_error = TRUE;
-	break;
+		MessageBoxDlg("֤ν˼Ԥޤ(ݥ顼)",
+					  NULL,
+					  MBF_TYPEOK|MBF_ICONSTOP);
+		return;
       }
+      writerp[writer_num].hid = g_WriterList[index].hid;
+      writerp[writer_num].tid = g_WriterList[index].tid;
       writer_num++;
     }
   }
+  else if(sel_writer == g_WriterISO){
+    writerp->hid = HID_VIRTUAL;
+    writerp->tid = CMDDRVTYPE_ISO;
+    writer_num = 1;
+  }
+  else if(sel_writer == g_WriterNet){
+    writerp->hid = HID_VIRTUAL;
+    writerp->tid = CMDDRVTYPE_NET;
+    writer_num = 1;
+  }
   else{
-    ret = InitializeDrive(&writer, 0, 50);
-    if(ret!=RET_OK){
-      init_error = TRUE;
-    }
-    else{
-      writerp = &writer;
-      writer_num = 1;
-      /* writerΥХåեreaderΥХåեͭ */
-      writer.data_buf = reader.data_buf;
-      writer.bufsize = reader.bufsize;
-      if(sel_writer != g_WriterISO){
-	ret = SetAspiSetting(&writer,
-			     &g_WriterList[sel_writer].hid,
-			     &g_WriterList[sel_writer].tid,
-			     &timeout);
-	if(ret!=RET_OK){
-	  init_error = TRUE;
-	}
-      }
-    }
+    writerp->hid = g_WriterList[sel_writer].hid;
+    writerp->tid = g_WriterList[sel_writer].tid;
+    writer_num = 1;
   }
-  if(init_error){
-    FreeDrive(&reader);
-    MessageBoxDlg("֤ν˼Ԥޤ", NULL,
-		  MBF_TYPEOK|MBF_ICONSTOP);
-    for(index=0; index<writer_num; index++){
-      writerp[index].bufsize = 0;
-      writerp[index].data_buf = NULL;
-      FreeDrive(&writerp[index]);
-    }
-    if(writerp != &writer){
-      free(writerp);
-    }
-    return;
+
+  /* ɹ */
+  if(sel_reader == g_ReaderISO){
+    reader.hid = HID_VIRTUAL;
+    reader.tid = CMDDRVTYPE_ISO;
   }
-  
+  else if(sel_reader == g_ReaderNet){
+    reader.hid = HID_VIRTUAL;
+    reader.tid = CMDDRVTYPE_NET;
+  }
+  else{
+    reader.hid = g_ReaderList[sel_reader].hid;
+    reader.tid = g_ReaderList[sel_reader].tid;
+  }
+
+
   UIClearAbort();
   g_bRunningCopy = TRUE;
   gtk_widget_set_sensitive(g_btnExit, FALSE);
@@ -224,64 +204,20 @@ on_btnStartCopy_clicked(GtkWidget *widget, gpointer user_data)
   gtk_widget_set_sensitive(g_optMenu2, FALSE);
   gtk_label_set_text(GTK_LABEL(GTK_BIN(g_btnStartCopy)->child), "");
 
-  ret = CopyDisc(sel_reader==g_ReaderISO ? NULL : &reader,
-		 sel_writer==g_WriterISO ? NULL : writerp,
-		 writer_num);
-  switch(ret){
-  case RET_ABORT:
-    mbret = MessageBoxDlg("Ǥޤ", NULL,
-			  MBF_TYPEOK|MBF_ICONINFORMATION);
-    break;
-  case RET_OK:
-    if(sel_writer==g_WriterISO){
-      mbret = MessageBoxDlg("˴λޤ", NULL,
-			    MBF_TYPEOK|MBF_ICONINFORMATION);
-    }
-    else{
-      mbret = MessageBoxDlg("˴λޤ\nȤޤ?",
-			    NULL,
-			    MBF_TYPEYESNO|MBF_ICONINFORMATION);
-    }
-    break;
-  default:
-    mbret = MessageBoxDlg("顼ȯޤ\nޥɥޤ?",
-			  NULL,
-			  MBF_TYPEYESNO|MBF_ICONSTOP);
-  }
-  if(mbret==MBR_YES){
-    if(ret!=RET_OK){
-      CreateLog(&reader, writerp, writer_num);
-    }
-    else if(sel_writer != g_WriterISO){
-      for(index=0; index<writer_num; index++){
-	OpenTray(&writerp[index]);
-      }
-    }
-  }
+  ret = CopyDisc(&reader, writerp, writer_num);
   
   gtk_widget_set_sensitive(g_btnStartCopy, TRUE);
   gtk_widget_set_sensitive(g_btnExit, TRUE);
   gtk_widget_set_sensitive(g_optMenu1, TRUE);
   gtk_widget_set_sensitive(g_optMenu2, TRUE);
   gtk_label_set_text(GTK_LABEL(GTK_BIN(g_btnStartCopy)->child), "ʣ̳");
+  g_bRunningCopy = FALSE;
   
-  for(index=0; index<writer_num; index++){
-    /* ͭƤХåեڤΥ */
-    writer.bufsize = 0;
-    writer.data_buf = NULL;
-    FreeDrive(&writerp[index]);
-  }
-  /* ɥ饤³ */
-  FreeDrive(&reader);
-
-  if(writerp != &writer){
-    free(writerp);
-  }
+  /* ֥ */
+  free(writerp);
 
   UIMeter1Initialize(NULL);
   UIMeter2Initialize(NULL);
-
-  g_bRunningCopy = FALSE;
 }
 
 void
