@@ -1,5 +1,6 @@
 #import "MainWindowControl.h"
 #import "SettingDlgControl.h"
+#import "SettingNetControl.h"
 #import "ui.h"
 #import "scan.h"
 #import "cmd.h"
@@ -14,6 +15,8 @@ static BOOL g_bRunningCopy=FALSE;
 static int g_AllWritableDrives=-1;
 static int g_ReaderISO=-1;
 static int g_WriterISO=-1;
+static int g_ReaderNet=-1;
+static int g_WriterNet=-1;
 
 - (IBAction)exitProgram:(id)sender
 {
@@ -48,6 +51,11 @@ static int g_WriterISO=-1;
 	[readerCombo addItemWithTitle: [NSString stringWithCString : "ISOC[Wt@C"]];
 	g_WriterISO = [writerCombo numberOfItems];
 	[writerCombo addItemWithTitle: [NSString stringWithCString : "ISOC[Wt@C"]];
+	
+	g_ReaderNet = [readerCombo numberOfItems];
+	[readerCombo addItemWithTitle: [NSString stringWithCString : "lbg[N"]];
+	g_WriterNet = [writerCombo numberOfItems];
+	[writerCombo addItemWithTitle: [NSString stringWithCString : "lbg[N"]];
 }
 
 - (IBAction)startCopy:(id)sender
@@ -77,44 +85,22 @@ static int g_WriterISO=-1;
 - (void)startCopyThread:(id)object
 {
 	NSAutoreleasePool *ap = [[NSAutoreleasePool alloc] init];
-	DRIVE reader, writer, *writerp=NULL;
+	DRIVEID reader, *writerp=NULL;
 	int sel_reader, sel_writer;
-	DWORD timeout = 5*60;	/* R}h^CAEgT */
 	int ret;
-	int mbret;
-	BOOL onedrive=FALSE;
 	int writer_num = 1;
-	BOOL init_error = FALSE;
 	int index;
 	
 	/* IhCu擾 */
 	sel_reader = [readerCombo indexOfSelectedItem];
 	sel_writer = [writerCombo indexOfSelectedItem];
 	
-	if(sel_writer != g_AllWritableDrives &&
-	   sel_writer != g_WriterISO &&
-	   sel_reader != g_ReaderISO){
-		if(g_Readers[sel_reader].hid == g_Writers[sel_writer].hid &&
-	       g_Readers[sel_reader].tid == g_Writers[sel_writer].tid){
-			onedrive=TRUE;
-		}
-	}
-	/* hCu\̏ */
-	memset(&reader, 0, sizeof(DRIVE));
-	memset(&writer, 0, sizeof(DRIVE));
-	ret = InitializeDrive(&reader, 0x10000, 50);
-	if(ret==RET_OK){
-		if(sel_reader != g_ReaderISO){
-			ret = SetAspiSetting(&reader,
-						&g_Readers[sel_reader].hid,
-						&g_Readers[sel_reader].tid,
-						&timeout);
-		}
-	}
-	if(ret!=RET_OK){
-		[self MessageBoxDlg : "ǍȕɎs܂B"
-						 title : NULL
-						 flags : MBF_TYPEOK|MBF_ICONSTOP];
+	/* upm */
+	writerp = (DRIVEID *)malloc(sizeof(DRIVEID));
+	if(writerp==NULL){
+		[self MessageBoxDlg : "ȕɎs܂B(mۃG[)"
+					  title : NULL
+					  flags : MBF_TYPEOK|MBF_ICONSTOP];
 		[buttonStartCopy setEnabled : YES];
 		[buttonQuit setEnabled : YES];
 		[readerCombo setEnabled : YES];
@@ -123,136 +109,76 @@ static int g_WriterISO=-1;
 		[ap release];
 		return;
 	}
-	if(onedrive){
-		memcpy(&writer, &reader, sizeof(DRIVE));
-		writerp = &writer;
+	
+	/* uݒ */
+	if(sel_writer == g_AllWritableDrives){
+		writer_num = 0;
+		for(index=0; strlen(g_Writers[index].inqdata)!=0; index++){
+			if(g_Readers[sel_reader].hid == g_Writers[index].hid &&
+			   g_Readers[sel_reader].tid == g_Writers[index].tid){
+				/* ǎ摕u͖ */
+				continue;
+			}
+			writerp = (DRIVEID *)realloc(writerp, (writer_num+1)*sizeof(DRIVEID));
+			if(writerp==NULL){
+				[self MessageBoxDlg : "ȕɎs܂B(mۃG[)"
+							  title : NULL
+							  flags : MBF_TYPEOK|MBF_ICONSTOP];
+				[buttonStartCopy setEnabled : YES];
+				[buttonQuit setEnabled : YES];
+				[readerCombo setEnabled : YES];
+				[writerCombo setEnabled : YES];
+				[buttonStartCopy setTitle : [NSString stringWithCString : "ʊJn"]];
+				[ap release];
+				return;
+			}
+			writerp[writer_num].hid = g_Writers[index].hid;
+			writerp[writer_num].tid = g_Writers[index].tid;
+			writer_num++;
+		}
+	}
+	else if(sel_writer == g_WriterISO){
+		writerp->hid = HID_VIRTUAL;
+		writerp->tid = CMDDRVTYPE_ISO;
+		writer_num = 1;
+	}
+	else if(sel_writer == g_WriterNet){
+		writerp->hid = HID_VIRTUAL;
+		writerp->tid = CMDDRVTYPE_NET;
 		writer_num = 1;
 	}
 	else{
-		if(sel_writer == g_AllWritableDrives){
-			writer_num = 0;
-			for(index=0; strlen(g_Writers[index].inqdata)!=0; index++){
-				if(g_Readers[sel_reader].hid == g_Writers[index].hid &&
-	                g_Readers[sel_reader].tid == g_Writers[index].tid){
-					continue;
-				}
-				writerp = (DRIVE *)realloc(writerp, (writer_num+1)*sizeof(DRIVE));
-				if(writerp == NULL){
-					init_error = TRUE;
-					break;
-				}
-				memset(&writerp[writer_num], 0, sizeof(DRIVE));
-				ret = InitializeDrive(&writerp[writer_num], 0, 50);
-				if(ret != RET_OK){
-					init_error = TRUE;
-					break;
-				}
-				/* writer̃obt@reader̃obt@L */
-				writer.data_buf = reader.data_buf;
-				writer.bufsize = reader.bufsize;
-				ret = SetAspiSetting(&writerp[writer_num],
-						 &g_Writers[sel_writer].hid,
-						 &g_Writers[sel_writer].tid,
-						 &timeout);
-				if(ret != RET_OK){
-					init_error = TRUE;
-					break;
-				}
-				writer_num++;
-			}
-		}
-		else{
-			ret = InitializeDrive(&writer, 0, 50);
-			if(ret != RET_OK){
-				init_error = TRUE;
-			}
-			else{
-				writerp = &writer;
-				writer_num = 1;
-				/* writer̃obt@reader̃obt@L */
-				writer.data_buf = reader.data_buf;
-				writer.bufsize = reader.bufsize;
-				if(sel_writer != g_WriterISO){
-					ret = SetAspiSetting(&writer,
-						  &g_Writers[sel_writer].hid,
-						  &g_Writers[sel_writer].tid,
-						  &timeout);
-					if(ret != RET_OK){
-						init_error = TRUE;
-					}
-				}
-			}
-		}
-		if(init_error){
-			[self MessageBoxDlg : "ȕɎs܂B"
-								   title : NULL
-								   flags : MBF_TYPEOK|MBF_ICONSTOP];
-			FreeDrive(&reader);
-			for(index=0; index<writer_num; index++){
-				writerp[index].bufsize = 0;
-				writerp[index].data_buf = NULL;
-				FreeDrive(&writerp[index]);
-			}
-			if(writerp != &writer){
-				free(writerp);
-			}
-			[buttonStartCopy setEnabled : YES];
-			[buttonQuit setEnabled : YES];
-			[readerCombo setEnabled : YES];
-			[writerCombo setEnabled : YES];
-			[buttonStartCopy setTitle : [NSString stringWithCString : "ʊJn"]];
-			[ap release];
-			return;
-		}
+		writerp->hid = g_Writers[sel_writer].hid;
+		writerp->tid = g_Writers[sel_writer].tid;
+		writer_num = 1;
 	}
-	ret = CopyDisc(sel_reader==g_ReaderISO ? NULL : &reader,
-				sel_writer==g_WriterISO ? NULL : writerp,
-				writer_num);
-	switch(ret){
-	case RET_ABORT:
-		mbret = [self MessageBoxDlg : "ʏ͒f܂B"
-								 title : NULL
-								 flags : MBF_TYPEOK|MBF_ICONINFORMATION];
-		break;
-	case RET_OK:
-		mbret = [self MessageBoxDlg : "ʏ͐ɏI܂Bro܂H"
-								 title : NULL
-								 flags : MBF_TYPEYESNO|MBF_ICONINFORMATION];
-		break;
-	default:
-		mbret = [self MessageBoxDlg : "G[܂BR}hOo͂܂H"
-								 title : NULL
-								 flags : MBF_TYPEYESNO|MBF_ICONSTOP];
+	
+	/* ǎ摕uݒ */
+	if(sel_reader == g_ReaderISO){
+		reader.hid = HID_VIRTUAL;
+		reader.tid = CMDDRVTYPE_ISO;
 	}
-	if(mbret==MBR_YES){
-		if(ret!=RET_OK){
-			CreateLog(&reader, writerp, writer_num);
-		}
-		else if(sel_writer != g_WriterISO){
-			for(index=0; index<writer_num; index++){
-				OpenTray(&writerp[index]);
-			}
-		}
+	else if(sel_reader == g_ReaderNet){
+		reader.hid = HID_VIRTUAL;
+		reader.tid = CMDDRVTYPE_NET;
 	}
+	else{
+		reader.hid = g_Readers[sel_reader].hid;
+		reader.tid = g_Readers[sel_reader].tid;
+	}
+
+	UIClearAbort();
+	ret = CopyDisc(&reader, writerp, writer_num);
 
 	[buttonStartCopy setEnabled : YES];
 	[buttonQuit setEnabled : YES];
 	[readerCombo setEnabled : YES];
 	[writerCombo setEnabled : YES];
 	[buttonStartCopy setTitle : [NSString stringWithCString : "ʊJn"]];
-	/* hCuڑ */
-	if(!onedrive){
-		for(index=0; index<writer_num; index++){
-			/* LĂobt@؂藣 */
-			writer.bufsize = 0;
-			writer.data_buf = NULL;
-			FreeDrive(&writerp[index]);
-		}
-	}
-	if(writerp != &writer){
-		free(writerp);
-	}
-	FreeDrive(&reader);
+
+	/* u */
+	free(writerp);
+
 	UIMeter1Initialize(NULL);
 	UIMeter2Initialize(NULL);
 	[ap release];
@@ -311,6 +237,18 @@ static int g_WriterISO=-1;
 	return [settingPanel openSettingDialog: option];
 }
 
+- (int)uiCtlNetSetting : (BOOL)serverMode
+		 remoteAddress : (char *)remote
+		   addressSize : (int)size
+			portNumber : (int *)portNumber
+{
+	return [netSettingPanel openNetSettingDialog:serverMode
+								   remoteAddress:remote
+									 addressSize:size
+									  portNumber:portNumber];
+}
+
+
 - (int)MessageBoxDlg : (const char *)messageKey
 			   title : (const char *)titleKey
 				flags : (int)flags
