@@ -137,3 +137,11 @@ int UIFileDialog(BOOL bOpen, char *filename, int size, const char *suffix)
 	
 	return UIDMRET_OK;
 }
+
+int UINetDialog(BOOL bServer, char *remote, int size, int *port_number)
+{
+	return [g_mwc uiCtlNetSetting : bServer
+					remoteAddress : remote
+					addressSize : size
+					   portNumber : port_number];
+}
