@@ -92,6 +92,8 @@ private:
 	int m_nAllWritableDrives;
 	int m_nReadDriveISO;
 	int m_nWriteDriveISO;
+	int m_nReadDriveNet;
+	int m_nWriteDriveNet;
 };
 
 //{{AFX_INSERT_LOCATION}}
