@@ -4,6 +4,7 @@
 #include "stdafx.h"
 #include "ui.h"
 #include "ThemeMessageDlg.h"
+#include "RemoteDlg.h"
 #include "EnbanFukusya.h"
 
 static BOOL g_bAbort=FALSE;
@@ -117,3 +118,25 @@ int UIFileDialog(BOOL bOpen, char *filename, int size, const char *suffix)
 	return UIDMRET_CANCEL;				
 }
 
+int UINetDialog(BOOL bServer, char *remote, int size, int *port_number)
+{
+	int nRetValue = UIDMRET_CANCEL;
+	CThemeSet *pThemeClass = ((CEnbanFukusyaApp *)AfxGetApp())->GetThemeClass();
+	CRemoteDlg *pDlg = new CRemoteDlg();
+	pDlg->SetTheme(pThemeClass);
+	pDlg->SetServerMode(bServer);
+	int nRet = pDlg->DoModal();
+	if(nRet == IDOK){
+		nRetValue = UIDMRET_OK;
+		if(bServer){
+			*port_number = pDlg->GetPortNumber();
+		}
+		else{
+			strncpy(remote, pDlg->GetRemoteAddress(), size);
+			remote[size-1] = '\0';
+			*port_number = pDlg->GetPortNumber();
+		}
+	}
+	delete pDlg;
+	return nRetValue;
+}
